#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <unistd.h>
#include <limits.h>
#include "coth.h"

#if defined(__amd_64__) || defined(__amd_64) || defined(__x86_64__) || defined(__x86_64)
#define X64 1
#endif

struct coth
{
#ifdef X64
   uint64_t register_store[15];
#else
   uint32_t register_store[7];
#endif
   unsigned char *stack_low;
   size_t stack_size;
};

coth_t *coth_main()
{
   coth_t *ctx;
   ctx= calloc(1,sizeof(*ctx));
  /* we already have a stack, so we don't need to do anything */
   return ctx;
}

coth_t *coth_new(void *faddr,size_t stack_size)
{
   coth_t *ctx;
   unsigned char *stack_high, *stack_low;
   unsigned int page_size;

   page_size= sysconf(_SC_PAGESIZE);

   ctx= malloc(sizeof(*ctx));

   if (stack_size<page_size) stack_size= page_size;
   if (stack_size%page_size) stack_size+= page_size-stack_size%page_size;
   stack_size+= page_size ;
   stack_low= mmap(0, stack_size,
        PROT_READ|PROT_WRITE,
        MAP_PRIVATE | MAP_ANONYMOUS | MAP_STACK,
        0,0);

   if (!stack_low) { free(ctx); return 0; }
   ctx->stack_low= stack_low;
   ctx->stack_size= stack_size;

   mprotect(stack_low, page_size, PROT_NONE);

   stack_high= ctx->stack_low + stack_size;

   /* now, initialize the fields in the register store so that 
      %ebp points to our new stack. it was the last one to be
      pushed, so register_store[0] is %ebp. */

#ifdef X64
   ctx->register_store[0]= (uint64_t) (stack_high - 16);

   /* we also put the function address at the top of the stack
      so that the return works properly. */
   *((void**) (stack_high-8))= faddr;
   *((void**) (stack_high-16))= (stack_high-16);

#else
   ctx->register_store[0]= (uint32_t) (ctx->stack_low + stack_size - 8);

   /* we also put the function address at the top of the stack
      so that the return works properly. */
   *((void**) (stack_high-4))= faddr;
   *((void**) (stack_high-8))= (stack_high-8);
#endif

   return ctx;
}

#ifdef X64
void coth_arg(coth_t *ctx, int index, void *value)
{
  int K;
  switch(index)
  {
  case 1: K= 1; break;  // rdi
  case 2: K= 2; break;  // rsi
  case 3: K= 3; break;  // rdx
  case 4: K= 4; break;  // rcx
  case 5: K= 7; break;  // r8
  case 6: K= 8; break;  // r9
  }
  ctx->register_store[K]= (uint64_t) value;
}
#endif

void coth_free(coth_t *ctx)
{
  munmap(ctx->stack_low,ctx->stack_size);
  free(ctx);
}
