#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include "ihex.h"


static uint16_t hex(uint8_t *s, int n,int *e)
{
   int i;
   uint16_t total= 0;
   *e= 0;
   for(i=0;i<n;i++)
   {
       if (s[i]>='0' && s[i]<='9') total= (total << 4) | (s[i]-'0');
       else if (s[i]>='a' && s[i]<='f') total= (total << 4) | (s[i]-'a'+10);
       else if (s[i]>='A' && s[i]<='F') total= (total << 4) | (s[i]-'A'+10);
       else { *e= 1; return 0; }
   }
   return total;
}

static char *mkerrmsg(const char *fmt, ...)
{
  va_list ap;
  char buf[2000];
  va_start(ap,fmt);
  memset(buf, 0, sizeof(buf));
  vsnprintf(buf, sizeof(buf), fmt, ap);
  va_end(ap);
  buf[sizeof(buf)-1]= 0;
  return strdup(buf);
}

#define HEXERR  if(e) goto hexerr
#define rhex_error(A) do { emsg= mkerrmsg A; goto finish; } while(0)


char* read_hex_file
  (char *file_name, uint16_t flashsize, uint8_t fill, uint8_t **Rdata)
{
  FILE *f;
  int i,k,e;
  uint8_t buffer[1024];
  uint8_t  *flash_data;
  char *emsg;
  int line;
  uint8_t *data;


  emsg= NULL;
  flash_data= NULL;
  f= NULL;

  if (!file_name) rhex_error(("internal error: bad arguments"));

  line= 0;
  f= fopen(file_name, "r");
  if (!f) rhex_error(("can't open %s: %s", file_name, strerror(errno)));

  
  flash_data= malloc(flashsize);
  memset(flash_data, fill, flashsize);

  while(fgets((char*) buffer,1024,f))
  {
    uint8_t *Z;
    uint16_t addr, len, code;
    uint8_t sum, byte;
    line++;
    sum= 0;
    if (buffer[0]!=':') 
        rhex_error(("%s:%d: line doesn't start with a ':', " "aborting.\n",
                      file_name, line));
    Z= buffer+1;
    len= hex(Z,2,&e); HEXERR;   Z+=2; sum+= len; 
    addr= hex(Z,4,&e); HEXERR;  Z+=4; sum+= (addr & 0xff) + (addr>>8); 
    code= hex(Z,2,&e); HEXERR; Z+=2; sum+= code; 
    if (code==1) goto finish;
    if (code!=0) continue; // we're only interested in data records
    if (addr+len>flashsize)
        rhex_error(("%s:%d: record with address=%04x len=%d "
          "goes out of limits, aborting.\n", file_name, line,addr,len));
    for(i=0;i<len;i++)
       { 
         byte= flash_data[addr+i]= hex(Z,2,&e); HEXERR;
         Z+= 2;
         sum+= byte;
       }
    sum+= hex(Z,2,&e); HEXERR;
    if (sum!=0)
        rhex_error(("%s:%d: invalid checksum, aborting.\n", file_name, line));
  }
  rhex_error(("%s: end of file record not found, aborting.\n", file_name));
finish:
  if (f) fclose(f);
  if (!emsg)
  {
    *Rdata= flash_data; 
    flash_data= NULL;
  }
  if (flash_data) free(flash_data);
  return emsg;
hexerr:
   rhex_error(("%s:%d: line is too short or has "
               "invalid characters.\n", file_name, line));
}


