#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <stddef.h>
#include <time.h>
#include <stdint.h>
#include <stdio.h>
#include "gfx.h"
#include "ticktock.h"
#include "worker.h"

int device_socket=-1;
static int tick_fd;

static int main_fd;
static void (*main_comp_func)();
uint32_t main_i1, main_i2;
void *main_p1, *main_p2;

static int bad_str(char *k)
{
  int i;
  if (!k || !k[0]) return 1;
  for(i=0;k[i];i++)
   if (k[i]<' ' || k[i]>=127 || k[i]=='"') return 1;
  return 0;
}

static int tick_event(gfx_event_t *ev)
{
  char c;
  if (ev->type!=GFXEV_READY_READ) return 0;
  switch(ev->key)
  {
  case 20: ticktock_ack(); worker_time_elapsed(100); break;
  case 30: worker_read_data(); break; 
  case 40: read(main_fd,&c,1);
           if (main_comp_func) {
             void (*f)(); f= main_comp_func;
             main_comp_func=NULL; (*f)(); }
           break;
  default: return 0;
  }
  return 1;
}


void ap_settings()
{
  char *s;
  char *ssid, *pwd;
  gfx_event_t event;
  gfx_AP_SETTING();
  
again:
  if (!gfx_event(NULL,&event)) goto again;
  if (tick_event(&event)) goto again;
  switch(event.type)
  {
  case GFXEV_QUIT: 
  case WapCANCEL: 
    break;
  case WapCHANGE:
    ssid= gfx_get_text(WapSSID);
    pwd= gfx_get_text(WapPWD);
    if (bad_str(ssid)) { gfx_logview_printf(Wlog,"Bad SSID\n"); goto again; }
    if (bad_str(pwd)) { gfx_logview_printf(Wlog,"Bad PWD\n"); goto again; }
    worker_run('A', ssid, pwd, 0, 0);
    break;
  default:
    goto again;
  }
  gfx_leave_screen();
}

void got_baud_rate()
{
  gfx_event_t event;
  int i,K, B;
  uint32_t rates[]= { 9600, 14400, 19200, 28800, 
                      38400, 57600, 76800, 115200, 0 };
  char buf[20];

  B= main_i1;
  if (B<0) return ;

  gfx_BAUD_SETTING();

  for(i=0;rates[i];i++)
  {
    sprintf(buf,"%d", rates[i]);
    gfx_insert(Wbaud, -1, buf, NULL, NULL);
    if (B==rates[i]) gfx_set_selected(Wbaud, i); 
  }

again:
  if (!gfx_event(NULL,&event)) goto again;
  if (tick_event(&event)) goto again;
  switch(event.type)
  {
  case GFXEV_QUIT: 
  case WbaudrateCANCEL: 
    K= -1;
    goto done;
  default:
    goto again;
  case WbaudrateOK: 
    K= gfx_get_selected(Wbaud);
    goto done;
  }
done:
  gfx_leave_screen();
  if (K<0 || K>=i) return ;
  gfx_logview_printf(Wlog, "Changing baud rate to %d\n",rates[K]);
  worker_run('B', NULL, NULL, rates[K], 0);
}


int main()
{
  int quit;
  char *str;
  char *s;
  srand(time(NULL));
  gfx_init(NULL,600,400,"WiFi Programmer");
  quit= 0;
  gfx_SCREEN_0();
  gfx_logview_printf(Wlog, "WiFi Programmer - 1.0\n"
                           "Copyright 2017 Sukru Cinar\n");
  main_fd= worker_init();
  if (main_fd<0)
  {
  gfx_logview_printf(Wlog, "Unable to init worker: %s\n",
             strerror(errno));
  }
  else
  {
    gfx_monitor_file(main_fd, 40, 1, 0, 0);
  }
  tick_fd= ticktock_begin(100);
  gfx_monitor_file(tick_fd, 20, 1, 0, 0);
  while(!quit)
  {
    gfx_event_t event;
    if (!gfx_event(NULL,&event)) continue;
    if (tick_event(&event)) continue;
    switch(event.type)
    {
    case GFXEV_QUIT: quit=1; break;
    case Wconnect:
      if (device_socket!=-1)
      {
        gfx_logview_printf(Wlog, "Already connected, disconnecting.\n");
        close_device(1);
      }
      str= gfx_get_text(Wip);
      if (!str || !str[0]) 
      {
        gfx_logview_printf(Wlog, "Bad address, won't connect.\n");
        break;
      }
      open_device(str);
      worker_run('I', 0, 0, 1, 0);
      break;
    case Wapsettings:
      ap_settings();
      break;
    case Wsetbaud:
     main_comp_func= got_baud_rate;
     worker_run('b', 0, 0, 0, 0);
     break;
  case Wtest:
#if 0
  s= gfx_get_text(Waddr);
  if (!s || !s[0]) { gfx_logview_printf(Wlog, "enter an address\n"); break;}
#endif
  worker_run('T', NULL, NULL, strtol(s, NULL, 0),0);
  break;
    }
  }
  ticktock_end();
  
  return 0;
}
