#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include "ticktock.h"

static pthread_t tick;
static unsigned int delay;
static int pipe_w, pipe_r;

void msleep(long msecs)
{
  struct timespec req,rem;
  req.tv_sec= msecs/1000;
  msecs %= 1000;
  req.tv_nsec= msecs * 1000 * 1000;
again:
  if (nanosleep(&req,&rem))
  {
    if (errno==EINTR) { req= rem; goto again; }
    sleep(1); // slow clock..
  }
}

static void* oscillate(void *_dummy)
{
  while(1)
  {
    char c= 'T';
    msleep(delay);
    if (write(pipe_w, &c, 1)<0) 
     { if (errno==EPIPE || errno==EBADF)
       pthread_exit(NULL); }
  }
}

int ticktock_begin(unsigned int _delay)
{
  int fds[2];
  signal(SIGPIPE, SIG_IGN);
  delay= _delay;
  
  if (pipe(fds)) return -1;
  pipe_r= fds[0];
  pipe_w= fds[1];
  if (pthread_create(&tick, NULL, oscillate, NULL))
  {
    int eno;
    eno= errno;
    close(fds[0]);
    close(fds[1]);
    errno= eno;
    return -1;
  }
  return pipe_r;
}

void ticktock_ack()
{
  char c;
  read(pipe_r, &c,1);
}

void ticktock_end()
{
  void *dummy;
  close(pipe_w);
  close(pipe_r);
  pthread_join(tick, &dummy);
}


