
uint32_t b_tx, b_rx;

void usart_init(uint32_t baud, int stopbits,int parity,int charsize)
{
  UCSRA= (1<<U2X);  // double speed, no multi-processor communication mode

  uint32_t divider;
  uint32_t F1,F2;

  divider= F_CPU/8/baud - 1;


  F1= (divider+1)*baud*8;
  F2= (divider+2)*baud*8;
  if (F2-F_CPU<F_CPU-F1) divider++;

  UBRRH = (divider >> 8 ) & 0x0f;
  UBRRL = (divider & 0xff);

  uint8_t cs0, cs1, cs2;
  switch(charsize)
  {
  case 5: cs2= 0; cs1= 0; cs0= 0; break;
  case 6: cs2= 0; cs1= 0; cs0= 1; break;
  case 7: cs2= 0; cs1= 1; cs0= 0; break;
  case 8: cs2= 0; cs1= 1; cs0= 1; break;
  case 9: cs2= 1; cs1= 1; cs0= 1; break;
  }
  cs2= 0; cs1= 1; cs0= 1; 

  uint8_t pm0,pm1;
  switch(parity)
  {
  case 0: pm1= 0; pm0= 0; break;
  case 1: pm1= 1; pm0= 1; break;  // odd
  case 2: pm1= 1; pm0= 0; break;  // even
  default: pm1= 0; pm0= 0; break;
  }

  uint8_t sto;
  if (stopbits==2) sto= 1; else sto= 0;

  UCSRC= (1<<URSEL) | (pm1<<UPM1) | (pm0<<UPM0) | (sto<<USBS)
         | (cs1<<UCSZ1) | (cs0<<UCSZ0) ;

  UCSRB= (1<<RXCIE) |  (1<<RXEN) | (cs2<<UCSZ2);

}

ISR(USART_RXC_vect)
{
  uint8_t D,N;
  D= UDR;
  N= (c_tail+1)%BUFFER_SIZE;

  if (N==c_head) N= c_tail;
    // c_head= (c_head+1)%BUFFER_SIZE;

  c_data[c_tail]= D;
  c_tail= N;

  b_rx++;
}

uint8_t *ser_buf;
int ser_olen;
int ser_sending;

ISR(USART_UDRE_vect)
{
#if 0
  uint8_t D;
  if (ser_olen==0)
  {
    UCSRB &= ~ ((1<<UDRIE) | (1<<TXEN));
    ihead= iadvance%BUFSIZ;
    ser_sending= 0;
    return ;
  }
  D= ser_buf[0];
  ser_buf++;
  ser_olen--;
  UDR= D;
#endif
}

void usart_send(uint8_t *buf, int len)
{
  if (!len) return ;
#if 0
  while(ser_sending) _delay_us(500);
  ser_sending= 1;
  ser_buf= buf;
  ser_olen= len; 
  UCSRB |= (1<<UDRIE) | (1<<TXEN);
#endif
  int i;
  UCSRB |= (1<<TXEN);
  for(i=0;i<len;i++)
  {
     while(!(UCSRA&(1<<UDRE))) ;
     UDR= buf[i];
     b_tx++;
  }
  while(!(UCSRA&(1<<UDRE))) ;
  UCSRB &= ~(1<<TXEN);
}

void usart_stop()
{
  UCSRB &= ~((1<<TXEN) | (1<<RXEN));
}


